<?php
# Copyright 2006 Todd Palino
#
# This file is part of phpMyRecipes.
#
# phpMyRecipes is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# phpMyRecipes is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# phpMyRecipes; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA

  function check_perms($cwd, $uid, $gid) {
    $err = 0;

    if ($contents = opendir($cwd)) {
      while (($node = readdir($contents)) !== false) {
        if (($node != ".") && ($node != "..")) {
          $fn = $cwd . "/" . $node;

          $f_uid = fileowner($fn);
          $f_gid = filegroup($fn);
          $f_perm = fileperms($fn);

          if ($f_uid == $uid) {
            # Set the perms ourselves
            if (is_dir($fn)) {
              if (! chmod($fn, 0755)) {
                print "$fn: Owner ($f_uid), Group ($f_gid), Permissions (" . substr(sprintf('%o', $f_perm), -4) . ") - Cannot set permissions to 0755.<BR>\n";
                $err = 1;
              }
            } else {
              if (! chmod($fn, 0644)) {
                print "$fn: Owner ($f_uid), Group ($f_gid), Permissions (" . substr(sprintf('%o', $f_perm), -4) . ") - Cannot set permissions to 0644.<BR>\n";
                $err = 1;
              }
            }
          } else {
            # Owner not OK
            print "$fn: Owner ($f_uid), Group ($f_gid), Permissions (" . substr(sprintf('%o', $f_perm), -4) . ") - Owner is incorrect.<BR>\n";
            $err = 1;
          }

          if (is_dir($fn)) {
            if (check_perms($fn, $uid, $gid)) {
              $err = 1;
            }
          }
        }
      }
    } else {
      print "Cannot open directory ($cwd) for reading.<BR>\n";
      $err = 1;
    }

    return $err;
  }

  function write_manual_instructions() {
?>
</P>

<P>Your files are not writeable by this upgrade script.  That means
that I can tell you what changes need to happen for the upgrade to
complete, but I can't make the changes for you.  This will involve
changes to 1 file.</P>

<P>Edit the common.php file.  First, add the following lines to the
bottom of the "CONFIGURATION" section at the top:
<BLOCKQUOTE><PRE># The maximum size, in bytes, of uploaded image files
define("MAX_IMAGE_SIZE", 100000);</PRE></BLOCKQUOTE>

Next, remove everything below the line that says "MAKING CHANGES BELOW
THIS LINE IS NOT RECOMMENDED".  Don't worry.  All of that stuff has
been moved to the functions.php file.</P>

<P>That's it.  Once that is done, you should make sure the install
subdirectory is not executable.  You'll have to use a command similar
to the following:
<BLOCKQUOTE><PRE>chmod -R 0000 /var/www/html/phpMyRecipes/install</PRE></BLOCKQUOTE></P>
<?php 	
    write_sql_instructions();
  }
  
  function write_sql_instructions() {
?>
</P>

<P>I'm having a problem updating the DB schema.  This could be
because the database user that you configured when you installed
phpMyRecipes does not have the appropriate permissions to alter
the tables in the database.  In order to complete the upgrade,
you will have to manually execute the following SQL statement
on your database server against the <?php DB_DB ?> database:
<BLOCKQUOTE><PRE>ALTER TABLE `recipes` ADD `description` TEXT NOT NULL, ADD `creator` INT UNSIGNED NOT NULL, ADD `editor` INT UNSIGNED NOT NULL, ADD `imagefile` VARCHAR( 50 ) NOT NULL, DROP INDEX `name_2`, ADD FULLTEXT `name_2` (`name`,`instructions`,`description`)</PRE></BLOCKQUOTE></P>
<?php 	
  }
  
?>
<HTML>
<HEAD>
<TITLE>phpMyRecipes Upgrade</TITLE>
</HEAD>

<BODY>
<P><H1><CENTER>phpMyRecipes Installation</CENTER></H1><BR>
<H2>Step 1 - File Permissions</H2></P>

<?php
  # First, get the current uid and group
  if (! touch("/tmp/phpMyRecipe.test")) {
    print "Temp directory is not writeable.  Cannot proceed.\n";
    exit(1);
  }

  $run_uid = fileowner("/tmp/phpMyRecipe.test");
  $run_gid = filegroup("/tmp/phpMyRecipe.test");
  unlink("/tmp/phpMyRecipe.test");

  print "<P>Running as user ID $run_uid<BR>\n";
  print "Running as group ID $run_gid</P>\n";
  
  # Get current directory
  $cwd = dirname(__FILE__);
  $pos = strrpos($cwd, "/install");
  if ($pos === false) {
    print "<P>Your distribution is not correct.  Please start over.</P>\n";
    exit();
  } else {
    $rootdir = substr($cwd, 0, $pos);
  }

  $doit = 0;
  print "<P>Checking file ownership and permissions:<BR>\n";
  if (check_perms($rootdir, $run_uid, $run_gid)) {
  	write_manual_instructions();
    exit();
  }
  
  print "Permissions are OK, continuing with upgrade.</P>\n";

  print "<P>Modifying $rootdir/common.php file<BR>\n";

  print "Moving file to backup<BR>\n";
  rename("$rootdir/common.php", "$rootdir/common_original.php");

  $rhandle = fopen("$rootdir/common_original.php", "r");
  $whandle = fopen("$rootdir/common.php", "w");
  if ($rhandle && $whandle) {
  	print "Reading original file, and writing new common.php<BR>\n";
  	$state = 0;
  	while (!feof($rhandle) && ($state < 3)) {
      $line = fgets($rhandle);
      if ($state == 0) {
        if (preg_match("/DB_HOST/", $line)) {
        	# Watch for our defines
        	fwrite($whandle, $line);
        	$state = 1;
        } else {
        	# Write it out
        	fwrite($whandle, $line);
        }
      } elseif ($state == 1) {
        if (preg_match("/^define/", $line)) {
        	# Eval the line to get our define
        	eval($line);
        	fwrite($whandle, $line);
        } elseif (preg_match("/^########/", $line)) {
        	# Print our new config value
        	fwrite($whandle, "# The maximum size, in bytes, of uploaded image files\n");
        	fwrite($whandle, "define(\"MAX_IMAGE_SIZE\", 100000);\n\n");
        	fwrite($whandle, $line);
        	$state = 2;
        } else {
        	# Write it out
        	fwrite($whandle, $line);
        }
      } elseif ($state == 2) {
        if (preg_match("/MAKING CHANGES/", $line)) {
        	# Remove everything else because it's in the functions file
        	fwrite($whandle, "?>\n");
        	$state = 3;
        } else {
        	# Write it out
        	fwrite($whandle, $line);
        }
      }
  	}
  	
    fclose($whandle);
    fclose($rhandle);
    print "Finished with common.php.</P>\n";
  } else {
  	print "Error opening common_original.php or common.php.</P>\n";
  	write_manual_instructions();
  	exit();
  }

  print "<P>Updating database schema<BR>\nConnecting to database - ";
  if (($dbconn = mysql_connect(DB_HOST, DB_USER, DB_PASS)) <= 0) {
    print "Failed<BR>\nCannot connect to database<BR>\nError -" . mysql_error() . "</P>\n";
  	write_sql_instructions();
  	exit();
  }
  if (! mysql_select_db(DB_DB, $dbconn)) {
    print "Failed<BR>\nCannot select database<BR>\nError -" . mysql_error() . "</P>\n";
  	write_sql_instructions();
  	exit();
  }
  print " OK<BR>\nAdding fields to recipes table - ";
  if (mysql_query("ALTER TABLE `recipes` ADD `description` TEXT NOT NULL, ADD `creator` INT UNSIGNED NOT NULL, ADD `editor` INT UNSIGNED NOT NULL, ADD `imagefile` VARCHAR( 50 ) NOT NULL, DROP INDEX `name_2`, ADD FULLTEXT `name_2` (`name`,`instructions`,`description`)", $dbconn)) {
  	print " OK<BR>\n";
  } else {
    print "Failed<BR>\nCannot alter table<BR>\nError - " . mysql_error() . "</P>\n";
  	write_sql_instructions();
  	exit();
  }

  # Make the installation scripts inaccessible
  print "<P>Making installation scripts inaccessible.</P>\n";
  chmod("./install.php", 0000);
  chmod("./step2.php", 0000);
  chmod("./step3.php", 0000);
  chmod("./step4.php", 0000);
  chmod("./upgrade.php", 0000);
  chmod(".", 0700);

?>
<P><H2>Upgrade completed!</H2></P>

</BODY>
</HTML>
